package logic

import (
	"encoding/json"
	"errors"
	"fusenapi/model/gmodel"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"

	"context"

	"fusenapi/server/home-user-auth/internal/svc"
	"fusenapi/server/home-user-auth/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
	"gorm.io/gorm"
)

type UserLogoListLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewUserLogoListLogic(ctx context.Context, svcCtx *svc.ServiceContext) *UserLogoListLogic {
	return &UserLogoListLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 处理进入前逻辑w,r
// func (l *UserLogoListLogic) BeforeLogic(w http.ResponseWriter, r *http.Request) {
// }

// 处理逻辑后 w,r 如:重定向, resp 必须重新处理
// func (l *UserLogoListLogic) AfterLogic(w http.ResponseWriter, r *http.Request, resp *basic.Response) {
// // httpx.OkJsonCtx(r.Context(), w, resp)
// }

func (l *UserLogoListLogic) UserLogoList(req *types.UserLogoListReq, userinfo *auth.UserInfo) (resp *basic.Response) {
	// 返回值必须调用Set重新返回, resp可以空指针调用 resp.SetStatus(basic.CodeOK, data)
	// userinfo 传入值时, 一定不为null
	// 定义用户ID
	var userId int64
	var guestId int64

	// 检查用户是否是游客
	if userinfo.IsGuest() {
		// 如果是，使用游客ID和游客键名格式
		guestId = userinfo.GuestId
	} else {
		// 否则，使用用户ID和用户键名格式
		userId = userinfo.UserId
	}

	userMaterialModel := gmodel.NewFsUserMaterialModel(l.svcCtx.MysqlConn)
	userMaterialRSB := userMaterialModel.RowSelectBuilder(nil).Preload("ResourceInfo", func(dbPreload *gorm.DB) *gorm.DB {
		return dbPreload.Table(gmodel.NewFsResourceModel(l.svcCtx.MysqlConn).TableName())
	}).Where("module = ?", "logo").Order("id desc").Limit(10)
	if userId != 0 {
		userMaterialRSB.Where("user_id = ?", userId)
	} else {
		userMaterialRSB.Where("guest_id = ?", guestId)
	}
	list, err := userMaterialModel.FindList(l.ctx, userMaterialRSB, nil, "")
	if err != nil {
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return resp.SetStatusWithMessage(basic.CodeDbRecordNotFoundErr, "data not found")
		}
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeServiceErr, "failed to get data list")
	}
	if len(list) > 0 {
		var merchantCategoryIds []int64
		for _, v := range list {
			var metadataMap map[string]interface{}
			if v.Metadata != nil {
				json.Unmarshal(*v.Metadata, &metadataMap)

				merchantCategory, merchantCategoryEix := metadataMap["merchant_category"]
				if merchantCategoryEix {
					merchantCategoryId := int64(merchantCategory.(float64))
					merchantCategoryIds = append(merchantCategoryIds, merchantCategoryId)
				}
				metadataMap["merchant_category_info"] = nil
				v.MetaDataMap = metadataMap
			}

			if v.ResourceInfo != nil {
				if v.ResourceInfo.Metadata != nil {
					var resourceMetadata map[string]interface{}
					json.Unmarshal([]byte(*v.ResourceInfo.Metadata), &resourceMetadata)
					v.ResourceInfo.MetaDataMap = resourceMetadata
				}
			}
		}
		if len(merchantCategoryIds) > 0 {
			newFsMerchantCategoryModel := gmodel.NewFsMerchantCategoryModel(l.svcCtx.MysqlConn)
			fsMerchantCategoryDb := newFsMerchantCategoryModel.BuilderDB(l.ctx, nil).Model(&gmodel.FsMerchantCategory{}).Where("id in ?", merchantCategoryIds)
			resfsMerchantCategoryDb, err := newFsMerchantCategoryModel.FindAll(fsMerchantCategoryDb, nil, "")
			if err != nil {
				logx.Error(err)
				return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "newFsMerchantCategoryModel FindAll db")
			}
			merchantCategoryDataLen := len(resfsMerchantCategoryDb)
			var merchantCategoryData = make(map[int64]*gmodel.FsMerchantCategory, merchantCategoryDataLen)
			if merchantCategoryDataLen > 0 {
				for _, v := range resfsMerchantCategoryDb {
					merchantCategoryData[v.Id] = v
				}
			}
			for _, v := range list {
				if v.MetaDataMap != nil {
					merchantCategory1, merchantCategoryEix1 := v.MetaDataMap["merchant_category"]
					if merchantCategoryEix1 {
						merchantCategoryId1 := int64(merchantCategory1.(float64))
						v.MetaDataMap["merchant_category_info"] = merchantCategoryData[merchantCategoryId1]
					} else {
						v.MetaDataMap["merchant_category_info"] = nil
					}
				}
			}
		}
	}
	return resp.SetStatus(basic.CodeOK, map[string]interface{}{
		"list": list,
	})
}
